package boids;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.AffineTransform;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class BoidPanel extends JPanel {

	private static final long serialVersionUID = -5007964443887883692L;
	private static final double[] SHAPE = { -5, -5, 0, 10, 5, -5, 0, -10 };
	private static final Point UP = new Point(0, -1);
	private static final Color BACKGROUND1 = new Color(130, 160, 225, 170);
	private static final Color BACKGROUND2 = new Color(40, 90, 180, 220);

	private Scene scene;
	private Point capture;
	private Point current;
	private Ray[] rays;
	private Image wallpaper;
	private Image scaled;
	private boolean renderBackground;

	class Ray {
		private final Color RAY_START = new Color(240, 240, 100, 0);
		private final Color RAY_END = new Color(210, 210, 130, 40);

		int width;
		double offset;
		double theta;

		public Ray(int width, double offset, double theta) {
			super();
			this.width = width;
			this.offset = offset;
			this.theta = theta;
		}

		void paint(Graphics g) {
			for (double th = theta - 0.04; th <= theta + 0.04; th += 0.02) {
				Graphics2D g2d = (Graphics2D) g;
				int height = 2 * getHeight();
				int centerx = (int) Math.round(getWidth() * offset);
				int centery = getHeight() / 2;

				java.awt.Rectangle r = new Rectangle(centerx - width / 2,
						centery - height / 2, width, height);

				AffineTransform tx = AffineTransform.getRotateInstance(th,
						centerx, 0);
				AffineTransform pt = g2d.getTransform();

				g2d.setTransform(tx);

				Paint pp = g2d.getPaint();
				GradientPaint p = new GradientPaint(0, height / 2, RAY_START,
						0, 0, RAY_END);
				g2d.setPaint(p);

				g2d.fill(r);

				g2d.setPaint(pp);
				g2d.setTransform(pt);
			}
		}
	}

	public BoidPanel(Scene scene) {
		this.scene = scene;
		this.setDoubleBuffered(true);

		rays = new Ray[6];
		double theta = Math.PI / 8;
		for (int offset = 0; offset < rays.length; ++offset)
			rays[offset] = new Ray(15, (double) (0.5 + offset) / rays.length,
					theta -= 0.1);

		try {
			if (wallpaper == null)
				wallpaper = ImageIO.read(new File("resources/ocean-floor.jpg"));
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent pointer) {
				current.setX(pointer.getX());
				current.setY(pointer.getY());
			}
		});

		addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent pointer) {
				capture = new Point(pointer.getX(), pointer.getY());
				current = new Point(pointer.getX(), pointer.getY());
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				fireDirectionChanged();
				capture = null;
				current = null;
			}
		});
	}

	public void addDirectionListener(DirectionListener l) {
		listenerList.add(DirectionListener.class, l);
	}

	protected void fireDirectionChanged() {
		Object[] listeners = listenerList.getListenerList();
		DirectionEvent event = null;
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == DirectionListener.class) {
				if (event == null)
					event = new DirectionEvent(capture, current);
				((DirectionListener) listeners[i + 1]).directionChanged(event);
			}
		}
	}

	@Override
	public void paint(Graphics graphics) {
		drawBackground(graphics);

		for (Boid b : scene.getAllBoids())
			drawBoid(graphics, b);

		if (renderBackground)
			for (Ray r : rays)
				r.paint(graphics);

		if (capture != null) {
			graphics.setColor(Color.BLACK);
			graphics.drawLine((int) capture.getX(), (int) capture.getY(),
					(int) current.getX(), (int) current.getY());
			graphics.setColor(Color.WHITE);
			graphics.drawOval((int) current.getX() - 5,
					(int) current.getY() - 5, 10, 10);
		}
	}

	private void drawBackground(Graphics graphics) {
		Graphics2D g2d = (Graphics2D) graphics;

		if (renderBackground) {
			if (scaled == null)
				scaled = wallpaper.getScaledInstance(getWidth(), getHeight(),
						Image.SCALE_DEFAULT);

			g2d.drawImage(scaled, 0, 0, getWidth(), getHeight(), null);
		}

		g2d.setPaint(new GradientPaint(0, 0, BACKGROUND1, 0, getHeight(),
				BACKGROUND2));
		graphics.fillRect(0, 0, getWidth(), getHeight());
	}

	public boolean isRenderBackground() {
		return renderBackground;
	}

	public void setRenderBackground(boolean renderBackground) {
		this.renderBackground = renderBackground;
	}

	public void componentResized(ComponentEvent paramComponentEvent) {
		scaled = wallpaper.getScaledInstance(getWidth(), getHeight(),
				Image.SCALE_DEFAULT);
	}

	private void drawBoid(Graphics graphics, Boid boid) {
		int x = (int) Math.round(boid.getPosition().getX());
		int y = getHeight() - (int) Math.round(boid.getPosition().getY());

		double dot = UP.dot(boid.getVelocity());
		double theta = Math.acos(dot / boid.getVelocity().norm());
		double[] shape = new double[SHAPE.length];
		AffineTransform tx = new AffineTransform();

		if (boid.getVelocity().getX() > 0)
			theta *= -1;
		tx.rotate(theta);

		tx.transform(SHAPE, 0, shape, 0, SHAPE.length / 2);

		int[] xs = { (int) Math.round(shape[0]) + x,
				(int) Math.round(shape[2]) + x, (int) Math.round(shape[4]) + x,
				(int) Math.round(shape[6]) + x };
		int[] ys = { (int) Math.round(shape[1]) + y,
				(int) Math.round(shape[3]) + y, (int) Math.round(shape[5]) + y,
				(int) Math.round(shape[7]) + y };

		graphics.setColor(boid.getParameters().getColor());
		graphics.fillPolygon(xs, ys, xs.length);
	}
}
