package boids;

import java.awt.Color;

/**
 * Holds the user-configurable parameters concerning a Boid.
 * 
 * @author Nir Atias
 *
 */
public class BoidParameters {

	/**
	 * Holds the default parameter values.
	 */
	public static final BoidParameters DEFAULT = new BoidParameters();
	
	private int minDistance = 40;
	private double returnRate = 0.7;
	private Color color = Color.BLACK;
	private double maxVelocity = 7;

	/**
	 * The distance to consider when avoiding collisions
	 * 
	 * @return The distance. Boids beyond this distance should be ignored.
	 */
	public int getMinDistance() {
		return minDistance;
	}

	/**
	 * The rate of returning/avoiding collisions.
	 * 
	 * @return The rate. This should be used when returning to the 
	 * scene's limits and when avoiding collisions.
	 */
	public double getReturnRate() {
		return returnRate;
	}

	/**
	 * The color of the boid. Used by the UI framework.
	 * 
	 * @return A color.
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * The maximum velocity that a Boid can travel.
	 *  
	 * @return The size of the maximum velocity.
	 */
	public double getMaxVelocity() {
		return maxVelocity;
	}

	/**
	 * Changes the minimum distance. 
	 * @param minDistance The new minimum distance
	 * @see BoidParameters#getMinDistance()
	 */
	public void setMinDistance(int minDistance) {
		this.minDistance = minDistance;
	}

	/**
	 * Changes the return rate.
	 * @param returnRate The new return rate.
	 * @see BoidParameters#getReturnRate()
	 */
	public void setReturnRate(double returnRate) {
		this.returnRate = returnRate;
	}

	/**
	 * Changes the color. 
	 * @param color The new color.
	 * @see BoidParameters#getColor()
	 */
	public void setColor(Color color) {
		this.color = color;
	}

	/**
	 * Changes the maximum velocity.
	 * 
	 * @param maxVelocity The new maximum velocity.
	 * @see BoidParameters#getMaxVelocity()
	 */
	public void setMaxVelocity(double maxVelocity) {
		this.maxVelocity = maxVelocity;
	}

	/**
	 * Default constructor. Used to build the DEFUALT parameters field.
	 */
	private BoidParameters() {
	}

	/**
	 * Constructs a new BoidParameters instance
	 * 
	 * @param minDistance The minimum distance, should be > 0
	 * @param returnRate The return rate, should be > 0
	 * @param color The color, should not be null
	 * @param maxVelocity The maximum velocity size, should be > 0
	 */
	/*
	 * @pre minDistance > 0
	 * @pre returnRate > 0
	 * @pre color != null
	 * @pre maxVelocity > 0 
	 */
	public BoidParameters(int minDistance, double returnRate, Color color,
			double maxVelocity) {
		this.minDistance = minDistance;
		this.returnRate = returnRate;
		this.color = color;
		this.maxVelocity = maxVelocity;
	}

}