package boids;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Random;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.UIManager;
import java.awt.Component;
import java.awt.FlowLayout;

/**
 * The main class for the Boids application. 
 * Run this class to start the UI.
 * 
 * @author Nir Atias
 *
 */
public class BoidsApplication {

	private static final int UPDATE_FREQUENCY = 20;
	private static final double MARGIN_FACTOR = 0.1;
	private JFrame frmBoidsApplication;
	private BoidPanel mainPanel;
	private Scene scene = new Scene();
	private Random rng = new Random();
	private JButton btnReset;
	private SimulationParametersPanel parametersPanel;
	private JPanel panel_2;
	private Component rigidArea;
	private JButton btnAddTestBoid;
	private JPanel panel_3;
	private JPanel panel_4;

	public BoidsApplication() {
		initialize();
	}

	// Initialize the UI
	private void initialize() {
		frmBoidsApplication = new JFrame();
		frmBoidsApplication.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent paramComponentEvent) {
				BoidPanel pane = mainPanel;
				int bottom = (int) Math.round(pane.getHeight() * MARGIN_FACTOR);
				int top = pane.getHeight() - bottom;
				int left = (int) Math.round(pane.getWidth() * MARGIN_FACTOR);
				int right = pane.getWidth() - left;
				scene.setLimits(new Rectangle(top, bottom, left, right));
			}
		});
		frmBoidsApplication.setTitle("Boids Application");
		frmBoidsApplication.setBounds(100, 100, 1019, 699);
		frmBoidsApplication.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmBoidsApplication.getContentPane().setLayout(new BorderLayout(0, 0));

		mainPanel = new BoidPanel(scene);
		frmBoidsApplication.getContentPane()
				.add(mainPanel, BorderLayout.CENTER);

		mainPanel.addDirectionListener(new DirectionListener() {
			@Override
			public void directionChanged(DirectionEvent event) {
				BoidPanel pane = mainPanel;

				event.getFrom().setY(pane.getHeight() - event.getFrom().getY());
				event.getTo().setY(pane.getHeight() - event.getTo().getY());

				createNewFlock(event.getFrom(), event.getDirection());
			}
		});

		JPanel configurationPanel = new JPanel();
		frmBoidsApplication.getContentPane().add(configurationPanel,
				BorderLayout.EAST);
		configurationPanel.setLayout(new BoxLayout(configurationPanel,
				BoxLayout.Y_AXIS));
		configurationPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		panel_2 = new JPanel();
		configurationPanel.add(panel_2);
		// panel_2.setLayout(new BoxLayout(panel_2, BoxLayout.X_AXIS));
		panel_2.setLayout(new BorderLayout());

		parametersPanel = new SimulationParametersPanel();
		parametersPanel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				mainPanel.setRenderBackground(parametersPanel
						.isShowBackground());
			}
		});

		panel_2.add(Box.createRigidArea(new Dimension(10, 0)),
				BorderLayout.EAST);
		panel_2.add(parametersPanel, BorderLayout.CENTER);
		panel_2.add(Box.createRigidArea(new Dimension(10, 0)),
				BorderLayout.WEST);

		configurationPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		panel_3 = new JPanel();
		configurationPanel.add(panel_3);
		panel_3.setLayout(new BoxLayout(panel_3, BoxLayout.X_AXIS));
		btnReset = new JButton("Reset");
		panel_3.add(btnReset);
		btnReset.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent paramActionEvent) {
				scene.reset();
			}
		});

		Timer t = new Timer(UPDATE_FREQUENCY, new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent paramActionEvent) {
				scene.update();
				mainPanel.repaint();
			}
		});

		parametersPanel.setFlockColor(generateRandomColor());

		rigidArea = Box.createRigidArea(new Dimension(0, 10));
		configurationPanel.add(rigidArea);

		panel_4 = new JPanel();
		configurationPanel.add(panel_4);
		panel_4.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

		btnAddTestBoid = new JButton("Add test boid");
		btnAddTestBoid.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				createTestBoid();
			}
		});
		panel_4.add(btnAddTestBoid);
		t.start();
	}

	private void createTestBoid() {
		BoidParameters parameters = getBoidParameters();

		Point at = new Point(rng.nextInt(mainPanel.getWidth()),
				rng.nextInt(mainPanel.getHeight()));
		Boid b = new Boid(at, generateRandomPoint((int) Math.round(parameters
				.getMaxVelocity() * .7)), parameters, scene);

		scene.addBoid(b);

		parametersPanel.setFlockColor(generateRandomColor());
	}

	private Point generateRandomPoint(int w, int h) {
		return new Point(rng.nextInt(2 * w) - w, rng.nextInt(2 * h) - h);
	}

	private Point generateRandomPoint(int w) {
		return generateRandomPoint(w, w);
	}

	private Color generateRandomColor() {
		return new Color(rng.nextInt(255), rng.nextInt(255), rng.nextInt(255));
	}

	private void createNewFlock(Point at, Point direction) {
		BoidParameters parameters = getBoidParameters();

		int size = parametersPanel.getNumberOfBoids();
		Boid[] boids = new Boid[size];

		for (int i = 0; i < size; ++i) {
			boids[i] = new Boid(at.plus(generateRandomPoint(10)),
					direction.plus(generateRandomPoint(3)), parameters, scene);
			scene.addBoid(boids[i]);
		}

		Flock f = new Flock(boids);
		f.setFlockingRate(parametersPanel.getFlockingRate());

		parametersPanel.setFlockColor(generateRandomColor());
	}

	private BoidParameters getBoidParameters() {
		BoidParameters parameters = new BoidParameters(
				parametersPanel.getMinDistance(),
				parametersPanel.getReturnRate(),
				parametersPanel.getFlockColor(),
				parametersPanel.getMaxVelocity());
		return parameters;
	}

	/**
	 * Run this method to get the Boids application running. 
	 * The application ignores command line parameters.
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Error setting native LAF: " + e);
		}

		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					BoidsApplication window = new BoidsApplication();
					window.frmBoidsApplication.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
}
